-- ScapeWorld-Funcs version 2.01
-- These are just some other functions and procedures which weren't initially
-- supported by Euphoria.   All were done by ScapeWorld!
-- ^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^
-- >>Warning!<< the procedures and functions in this file were placed in a
-- specific working order.   Changing the order of any procedures and functions
-- in this file may cause other procedures or functions in this file to
-- mal-function.   Altering any procedures or functions in this file may also
-- lead to their mal-function or discontinuant workability.
-- vvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvv

without type_check
include graphics.e
include image.e
include machine.e
include file.e
include get.e

---------------------------------------<Absolute>-------------------------------

-- Returns the absolute value of object 'a'.

global function abs(object a)
    return sqrt(a*a)
end function

---------------------------------------<Mod>------------------------------------

-- Returns all 'a' less than 'b'. When 'a'>= any 'integer multiple of 'b'
-- returns 'a'-'b'*'integer multiple of b'.

global function mod(object a,object b)
    return ((a/b)-floor(a/b))*b
end function

global function rem(object a,object b)
    return remainder(a,b)
end function

---------------------------------------<Sign>-----------------------------------

-- Returns the sign of 'a', or if 'a'=0 then returns 0.

global function sign(atom a)
    if a>0 then
	return 1
    elsif a=0 then
	return 0
    else
	return -1
    end if
end function

--------------------------------------<Integer>---------------------------------

-- Returns object 'a' rounded-off to the nearest integer value.

global function int(object a)
    return floor(a+.5)
end function

--------------------------------------<Limit>-----------------------------------

-- Returns 'a' within the range 'b' to 'c'

global function limit(atom a,atom b,atom c)
    if a<b then return b elsif a>c then return c else return a end if
end function

--------------------------------------<Spin>------------------------------------

-- Returns a two element array representing point {x,y} rotated 'a' radiants
-- about a center point {0,0}.
atom hpi hpi=3.14/2
global function spin(atom a,atom x,atom y)
    a=a+hpi return {sin(a)*x-cos(a)*y,sin(a)*y+cos(a)*x}
end function

---------------------------------------<Play>----------------------------------

-- Plays a sound of frequency s[x][1] for a duration of s[x][2].
-- The frequency if it is not already will be converted to an integer, but the
-- duration may be a non-integer and is measured in seconds.
-- 'x' represents the number of the sound to be played in the sequence 's'.
-- eg: play({{100,10},{2,.5},{.1,5}})
-- If the frequency is 0 then it will pause for the duration set.
-- This plays a sound of frequency 100 for duration 10 followed by freq. 2 dur.
-- .5, and freq. .1 dur. 5.

atom t
global procedure play(sequence s)
 for z=1 to length(s)  do
     sound(int(abs(s[z][1])))
     t=time()
     while time()<t+abs(s[z][2]) do end while
     sound(0)
 end for
end procedure

--------------------------------------<Circle>----------------------------------

-- Draws a circle or ellipse of colour 'a' which is filled if 'b'>0, or if
-- 'b'=0 its not filled.   The x radius is 'xr' and the y radius is 'yr'.
-- The sequence 'f' contains the co-ordinates for the center of the circle.
-- The horizontal position from left to right across the screen is 'f[1]'.
-- The vertical position from top to bottom down the screen is 'f[2]'.

global procedure circle(atom a,atom b,atom xr,atom yr,sequence f)
    ellipse(int(a),abs(sign(b)),{f[1]-xr/2,f[2]-yr/2},{f[1]+xr/2,f[2]+yr/2})
end procedure

-----------------------------------<Line3D>-------------------------------------

-- Draws a 3d line on a cartesian plane using a set or multiple sets of x,y,z
-- co-ordinates.
-- The sequence 'ln' contains sets of x,y,z points.   The x co-ordinates are
-- represented by 'ln[n][1]', y co-ordinates are represented by 'ln[n][2]', and
-- z co-ordinates are represented by 'ln[n][3]'; in 'ln[n]', 'n' represents the
-- number of the set.
-- If there is only one co-ordinate specified it will use the 'circle()'
-- procedure instead, to draw a single point in free space, this reduces memory
-- use.   The sequence 'circl' has three elements, the first specifies whether
-- to fill or not, the second and third specify the horizontal and vertical
-- radii, respectively.   These have no effect if 'ln' contains more than one
-- sequence, but must still be specified otherwise an error will occur.
-- The colour of the 3d-line is 'colr'.
-- The perspective zoom magnitude is 'zoom'.
-- The maximum x axis length is 'maxx'.
-- The maximum y axis length is 'maxy'.
-- These are usually defined according to the horizontal and vertical dimensions
-- of the current screen mode, or are defined by the size of the viewport/window
-- being used.

atom v3d,w3d,x3d,y3d
global procedure line3d(sequence ln,atom colr,atom zoom,sequence circl,
 atom maxx,atom maxy)
 if sequence(ln) then
     if length(ln)=1 then
	 v3d=(ln[1][1]*zoom)/(ln[1][3]+zoom)
	 w3d=(ln[1][2]*zoom)/(ln[1][3]+zoom)
	 if circl[2]=.5 and circl[3]=.5 then
	     pixel(colr,{maxx/2+v3d,maxy/2-w3d})
	 else
	     circle(colr,circl[1],circl[2],circl[3],{maxx/2+v3d,maxy/2-w3d})
	 end if
     elsif length(ln)>1 then
	 for t=1 to length(ln)-1 do
	     v3d=(ln[t][1]*zoom)/(ln[t][3]+zoom)
	     w3d=(ln[t][2]*zoom)/(ln[t][3]+zoom)
	     x3d=(ln[t+1][1]*zoom)/(ln[t+1][3]+zoom)
	     y3d=(ln[t+1][2]*zoom)/(ln[t+1][3]+zoom)
	     draw_line(int(colr),{{maxx/2+x3d,maxy/2-y3d},{maxx/2+v3d,maxy/2
	     -w3d}})
	 end for
     end if
 end if
end procedure

----------------------------------<coord3D>-------------------------------------

-- Returns the colour value of a 3D co-ordinate in an image.

global function coord3d(atom x,atom y,atom z,atom zoom,sequence img)
    return img[mod((y*zoom)/(z+zoom),length(img))+1]
	      [mod((x*zoom)/(z+zoom),length(img[1]))+1]
end function

--------------------------------------<Pause>-----------------------------------

-- Stops and waits for key 'a' to be pressed.
-- If 'a'=-1 then it stops and waits for any key to be pressed.
-- If 'b' does not equal 0 then it beeps, otherwise it doesn't beep.

global procedure pause(atom a,atom b)
    if b!=0 then
	play({{1000,.01}})
    end if
    if a=-1 then
	while get_key()=-1 do
	end while
    else
	while get_key()!=a do
	end while
    end if
end procedure

--------------------------------------<Box>-------------------------------------

-- Draws a rectangular box of colour 'a', filled if 'b' is not 0 with a top-left
-- corner of 'c' and bottom-right corner of 'd'.

-- object e
-- global procedure box(atom a,atom b,sequence c,sequence d)
--  if abs(b)>0 then
--      e=repeat(repeat(a,d[1]-c[1]),d[2]-c[2])
--      display_image({c[1],c[2]},e)
--  else
--      e=repeat({a},d[2]-c[2])
--      pixel(repeat(a,d[1]-c[1]),{c[1],c[2]})
--      pixel(repeat(a,d[1]-c[1]),{c[1],d[2]})
--      display_image({c[1],c[2]},e)
--      display_image({d[1],c[2]},e)
--  end if
-- end procedure

---------------------------------------<Screen>---------------------------------

-- Sets the screen graphics mode to 'a'.

object scr
global procedure screen(atom a)
    scr=graphics_mode(a)
end procedure

--------------------------------------<High>------------------------------------

-- Returns the highest atom in sequence 'a'.

object hi
global function high(sequence a)
    for b=1 to length(a) do
	if sequence(a[b]) then
	    hi=high(a[b])
	else
	    if b=1 then
		hi=a[1]
	    end if
	    if a[b]>hi then
		hi=a[b]
	    end if
	end if
    end for
    return hi
end function

--------------------------------------<Low>-------------------------------------

-- Returns the lowest atom in sequence 'a'.

object lo
global function low(sequence a)
    for b=1 to length(a) do
	if sequence(a[b]) then
	    lo=low(a[b])
	else
	    if b=1 then
		lo=a[1]
	    end if
	    if a[b]<lo then
		lo=a[b]
	    end if
	end if
    end for
    return lo
end function
----------------------------------------<Cycle>---------------------------------

-- Returns sequence 'a' with it's 'c' dimension contents cycled forward 'b'
-- positions.
-- If 'b' is greater than the length of 'a' then the remainder of 'b' over 'a'
-- will be used.

object d,e,f
global function cycle(sequence a,atom b,atom c)
    f=-1
    if c=1 then
	if length(a)<=1 then
	    return a
	else
	    d=mod(b,length(a))
	    e=a[length(a)-d+1..length(a)]&a[1..length(a)-d]
	    return e
	end if
    else
	for d=1 to length(a) do
	    a[d]=cycle(a[d],b,c-1)
	end for 
	return a
    end if
end function

------------------------------------<Replace_in>--------------------------------

-- Replaces 'a' with 'b' in 'c' if 'a' exists in 'c', otherwise returns 'c'.

-- object pos
-- global function replace_in(object a,object b,sequence c)
--  if match(a,c)=0 then
--      return c
--  else
--      while 1 do
--          pos=match(a,c)
--          if pos=0 then exit end if
--          if pos=1 then
--              c=b&c[length(b)-2..length(c)]
--          elsif pos=length(c) then
--              c=c[1..pos-1]&b
--          else
--              c=c[1..pos-2]&b&c[length(b)-2..length(c)]
--          end if
--      end while
--      return c
--  end if
-- end function
-- puts(1,replace_in("later","soon","Good-bye and see you later!"))
-- pause(-1,0)
-----------------------------------<Numgrad>-----------------------------------

-- Returns a one-dimentional sequence filled with numbers, starting from 0 and
-- increasing by 'b' until 'a'-1 is reached.
-- If 'a' is not positive, a one-dimentional sequence of zeros of length equal
-- to absolute 'a' is returned.

sequence numg
global function numgrad(atom a,atom b)
    if a<0 then
	numg=repeat(0,abs(a))
	return numg
    else 
	numg=repeat(0,abs(a)/b)
	for c=0 to a-1 by b do
	    numg[c+1]=c
	end for
	return numg
    end if
end function

--------------------------------------<Sum>-------------------------------------

-- Returns an atom equivilant to the sum of all the elements in sequence 'a'
-- and all its sub-dimensions.

-- object summ
-- summ=0
-- global function sum(sequence a)
--  for b=1 to length(a) do
--      if sequence(a[b]) then
--          summ=sum(a[b])
--      else 
--          summ=summ+(a[b])
--      end if
--  end for
--  return summ
-- end function

--------------------------------------<Rotate>----------------------------------

-- Returns the sequence of image 'a' rotated 'b' degrees clockwise using 'x' &
-- 'y' as the center point for the rotation.
-- If 'scal' is a sequence then 'scal[1]' represents the horizontal resize of
-- the image, and 'scal[2]' represents the vertical resize of image.
-- If 'scal' is an atom then it represents the entire resize of the image
-- (1:1) ratio.

object cb,sb,horx,very,rotimg
global function rotate(sequence a,atom b,atom x,atom y,object scal)
    rotimg=a
    b=(b+90)*(3.142/180)
    cb=cos(b)
    sb=sin(b)
    for ver=1 to length(a) do
	for hor=1 to length(a[1]) do
	    if sequence(scal) then
		horx=sb*(hor-x)*scal[1]-cb*(ver-y)*scal[2]
		very=sb*(ver-y)*scal[2]+cb*(hor-x)*scal[1]
	    else
		horx=(sb*(hor-x)-cb*(ver-y))*scal
		very=(sb*(ver-y)+cb*(hor-x))*scal
	    end if
	    rotimg[ver][hor]=a[mod(very,length(a))+1][mod(horx,length(a[1]))+1]
	end for
    end for
    return rotimg
end function

---------------------------------------<Flip>-----------------------------------

-- Returns sequence 'a' that has been rotated 90 degrees either clockwise if b=0
-- otherwise counter-clockwise.
-- It's best to assign the result to an object if dimensions are not clear.

sequence flp
global function flip(sequence a,atom b)
    flp=repeat(repeat(0,length(a)),length(a[1]))
    if b=0 then
	for hor=1 to length(a[1]) do
	    for ver=1 to length(a) do
		flp[length(a[1])-hor+1][ver]=a[ver][hor]
	    end for
	end for
    else
	for hor=1 to length(a[1]) do
	    for ver=1 to length(a) do
		flp[hor][length(a)-ver+1]=a[ver][hor]
	    end for
	end for
    end if
    return flp
end function

-------------------------------------<Reverse>----------------------------------

-- Returns sequence 'a' with its 'b' dimension in reverse.

object rev
global function reverse(sequence a,atom b)
    if b = 1 then
	if length(a)<=1 then
	    return a
	else
	    rev=repeat(0,length(a))
	    for d=1 to length(a) do
		rev[d]=a[length(a)-d+1]
	    end for
	    return rev
	end if
    else
	for d=1 to length(a) do
	    a[d]=reverse(a[d],b-1)
	end for 
	return a
    end if
end function

-------------------------------------<Sprite>-----------------------------------

-- Pastes the 'sprite' at position 'x' and 'y',with transparency 'tr' in image
-- 'img', then returns 'img'.
-- Crops 'sprite' to fit in selected area of 'img'.

global function sprite(sequence img,sequence sprite,atom x,atom y,atom tr)
object ij,sw,sh,iw,ih
sw=length(sprite[1]) sh=length(sprite) iw=length(img[1]) ih=length(img)
if x>iw or x+sw<0 or y>ih or y+sh<0 then --abort(0) end if
return img end if
--> Vertical cropping:
if x=0 then x=1 end if if y=0 then y=1 end if
if sh>ih-y or y<0 then
    if sh>ih-y then -- Sprite protrudes past the top of the image
	sh=ih-y -- redefine sprite's height
	sprite=sprite[1..sh] -- crop the sprite's top
	ij=img[y..y+sh] -- collumn parallel with the sprite vertically
    end if
    if y<0 then -- Sprite protrudes past the bottom of the image
	sprite=sprite[-y..sh] -- crop the sprite's bottom
	sh=sh+y -- redefine sprite's height
	ij=img[1..sh] -- collumn parallel with the sprite vertically
    end if
else
    ij=img[y..y+sh] -- collumn parallel with the sprite vertically
end if
--> Horizontal cropping:
if sw>iw-x or x<0 then
    if sw>iw-x then -- Sprite protrudes past the right-side of the image
	sw=iw-x -- redefine sprite's width
	for ys=1 to sh do -- loop through sprite's rows
	    sprite[ys]=sprite[ys][1..sw] -- crop the sprite's right-side
	    ij[ys][x..x-1+sw]=sprite[ys]
	end for
    end if
    if x<0 then -- Sprite protrudes past the left-side of the image
	for ys=1 to sh do -- loop through sprite's rows
	    sprite[ys]=sprite[ys][-x..sw] -- crop the sprite's left-side
	    ij[ys][1..1+x+sw]=sprite[ys]
	end for
    end if
else
    for ys=1 to sh do -- loop through sprite's rows
	ij[ys][x..x-1+sw]=sprite[ys]
    end for
end if
--> Transparency processing:
if tr=100 then if y<0 then img[1..sh]=ij else img[y..y+sh]=ij end if
else if y<0 then img[1..sh]=((100-tr)*img[1..sh]+tr*ij)/100
else img[y..y+sh]=((100-tr)*img[y..y+sh]+tr*ij)/100 end if
end if
return img
end function

-----------------------------------<SpriteTl>-----------------------------------

-- Pastes the 'sprite' at position 'x' and 'y',with transparency 'tr' and
-- translucency with range(1-256) 'tl' in image 'img', then returns 'img'.
-- Crops 'sprite' to fit in selected area of 'img'.

global function spritetl(sequence img,sequence sprite,sequence tl,atom x,atom y,atom tr)
object ij,sw,sh,iw,ih
sw=length(sprite[1]) sh=length(sprite) iw=length(img[1]) ih=length(img)
if x>iw or x+sw<0 or y>ih or y+sh<0 then --abort(0) end if
return img end if
--> Vertical cropping:
if x=0 then x=1 end if if y=0 then y=1 end if
if sh>ih-y or y<0 then
    if sh>ih-y then -- Sprite protrudes past the top of the image
	sh=ih-y -- redefine sprite's height
	sprite=sprite[1..sh] -- crop the sprite's top
	tl=tl[1..sh]
	ij=img[y..y+sh] -- collumn parallel with the sprite vertically
    end if
    if y<0 then -- Sprite protrudes past the bottom of the image
	sprite=sprite[-y..sh] -- crop the sprite's bottom
	tl=tl[-y..sh]
	sh=sh+y -- redefine sprite's height
	ij=img[1..sh] -- collumn parallel with the sprite vertically
    end if
else
    ij=img[y..y+sh] -- collumn parallel with the sprite vertically
end if
--> Horizontal cropping:
if sw>iw-x or x<0 then
    if sw>iw-x then -- Sprite protrudes past the right-side of the image
	sw=iw-x -- redefine sprite's width
	for ys=1 to sh do -- loop through sprite's rows
	    sprite[ys]=sprite[ys][1..sw] -- crop the sprite's right-side
	    tl[ys]=tl[ys][1..sw]
	    ij[ys][x..x-1+sw]=(sprite[ys]*tl[ys]+ij[ys][x..x-1+sw]*(256-tl[ys]))/256
	end for
    end if
    if x<0 then -- Sprite protrudes past the left-side of the image
	for ys=1 to sh do -- loop through sprite's rows
	    sprite[ys]=sprite[ys][-x..sw] -- crop the sprite's left-side
	    tl[ys]=tl[ys][-x..sw]
	    ij[ys][1..1+x+sw]=(sprite[ys]*tl[ys]+ij[ys][1..1+x+sw]*(256-tl[ys]))/256
	end for
    end if
else
    for ys=1 to sh do -- loop through sprite's rows
	ij[ys][x..x-1+sw]=(sprite[ys]*tl[ys]+ij[ys][x..x-1+sw]*(256-tl[ys]))/256
    end for
end if
--> Transparency processing:
if tr=100 then if y<0 then img[1..sh]=ij else img[y..y+sh]=ij end if
else if y<0 then img[1..sh]=((100-tr)*img[1..sh]+tr*ij)/100
else img[y..y+sh]=((100-tr)*img[y..y+sh]+tr*ij)/100 end if
end if
return img
end function

--------------------------------------<Rndsel>----------------------------------

-- Returns a two element sequence of which the first element contains every
-- element of 'a' and it's sub-dimensions except a randomly selected element
-- which is stored in the second element of the returned sequence.

-- function rndsel(sequence a)
--  return
-- end function

--------------------------------------<Getn>------------------------------------

-- If 'n' is 1 returns 1 byte, otherwise returns a sequence 'n' elements long
-- starting from the current position in file 'fname'.

global function getn(object fname,atom n)
    object res
    if n=1 then
	res=getc(fname)
    else
	res=repeat(0,n)
	for i=1 to n do
	    res[i]=getc(fname)
	end for
    end if
    return res
end function

-------------------------------------<Ball>-------------------------------------

-- Returns an image with a circular cilloette on it.
-- > 'img' is the image to be used.
-- > 'x' represents the x-co-ordinate of the center of the circle.
-- > 'y' represents the y-co-ordinate of the center of the circle.
-- > 'w' represents the x-radius of the circle.
-- > 'h' represents the y-radius of the circle.

object im,lx,ly,cr,cp
global function ball(sequence img,atom x,atom y,atom w,atom h)
    ly=length(img)
    lx=length(img[1])
    im=img
    if h<w then cr=h cp=h/w else cr=w cp=w/h end if
    for hr=1 to ly do for wr=1 to lx do
    im[hr][wr]=sqrt(power((wr-x),2)*h/w*cp+power((hr-y),2)*w/h*cp)/cr
    if im[hr][wr]<1 then im[hr][wr]=1 else im[hr][wr]=0 end if
    end for end for return im*img
end function

--------------------------------------------------------------------------------
--<<------------------------- Total function count=21 ---------------------->>--
--<<------------------------- Total procedure count=6 ---------------------->>--
-- All Rights Reserved 1996-Eternity.
-- Copyright ScapeWorld! Enterprises Inc. Pty. Ltd.
-- For permissions please contact us: E-mail us at scapeworld@yahoo.com.
-- You want a copy of our projects and would like chat with either of us about
-- more profound ideas and concepts then E-mail us.
-- karnij's personal E-mail is at karnij666@yahoo.com.
--<<--<<--<<--<<--<<--<<--<<--<<--<<--<<---->>-->>-->>-->>-->>-->>-->>-->>-->>--
--------------------------------------------------------------------------------
-- EOF (End-Of-File)
